//-*-C++-*- Header file for the AudioPanel Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _AudioPanel_h_
#define _AudioPanel_h_ 1
#include "pAudio.h"
#include "pControl.h"
#include "pSpecial.h"
#include "pDisplay.h"
#include "AudioDev.h"
#include "APDialog.h"



/**
  The {\bf AudioPanel} object is a Qt-widget that creates a spiffy
  audio control panel with lots and lots of options.  There are
  methods for setting various colors and fonts, as well as two
  different sizes and two styles of labelling.

  Rather than try to place all of the necessary Qt-components on its
  own, {\bf AudioPanel} uses 4 sub-widgets, all of which are private
  members: \Ref{pAudio}, \Ref{pControl}, \Ref{pSpecial}, and
  \Ref{pDisplay}.  {\bf AudioPanel} contains several methods that take
  care of the necessary signal-slot connections.  All connections go
  to an \Ref{AudioDev} object or an \Ref{APDialog}, so that any audio
  device can use the panel, not just CDROM drives!  There is even
  flexibility in some of the button icons.

  If you decide to use the {\bf AudioPanel} in a program of your own,
  follow these tips:

  \begin{enumerate}
  
  \item In the constructor, \Ref{AudioPanel}, be sure to set the
  parameter {\bf useEject} appropriately.  If your program plays files
  [sound, mpeg, etc.], set this to {\em false}. This replaces the
  "eject" icon on the \Ref{loadeject} button with a file folder.

  \item Connect \Ref{loadeject} to a slot:
  \begin{itemize}
  \item If your program plays "hard media" [like a CD], connect this
  to the \Ref{AudioDev} slot that controls the device eject mechanism.
  \item If your program plays files, connect this slot to some {\bf
  QObject} derived from a file browser.
  \end{itemize}

  \item Create an object derived from \Ref{AudioPanel} to control your
  player device.

  \item Use \Ref{pluginAudioControl} and \Ref{pluginDisplayChange} to
  automagically connect all of the buttons and displays on the {\bf
  AudioPanel} to your device signals.

  \item Use \Ref{pluginPanels} to connect the {\bf AudioPanel} to
  special panel objects derived from an \Ref{APDialog} object.
  \begin{itemize}
  \item You can use the \Ref{MaintainDB}, \Ref{Config} and
  \Ref{Program} objects, or you can create your own.
  \item I strongly suggest you use \Ref{MaintainDB}, since it
  automagically interfaces to {\tt WorkMan}-style and {\tt Xmcd}-style
  databases.
  \end{itemize}

  \end{enumerate}

  Check out the documentation for other member methods to see how to
  use them.

  I would really prefer that you {\em not} modify the internals of
  this class, or that of \Ref{APDialog}, \Ref{pAudio}, \Ref{pControl},
  \Ref{pSpecial}, and \Ref{pDisplay}.  [The one exception is, of
  course, bugfixes.]  If you really, {\em really} need access to any
  of the signals and slots of \Ref{pAudio}, \Ref{pControl},
  \Ref{pSpecial}, or \Ref{pDisplay}, I'd prefer that you contact me.
  I'll make modifications and add a set of "wrapper" functions to
  access the desired objects.

  {\bf
  \begin{verbatim}
  
  ----------------------------------------------------------
  ----------------------------------------------------------
  \end{verbatim}
  }

  @memo Consistent, spiffy audio control panel.
  @see AudioDev, APDialog, pAudio, pControl, pSpecial, pDisplay

*/
class AudioPanel : public QWidget
{
	 //
	 Q_OBJECT

   public:
	 /**
	   Class Constructor.  You can specify several parameters to
	   control its appearance.

	   @param parent The parent widget.  Most likely a null pointer
	   [{\em i.e} the desktop is the parent].
	   @param dn A string containing the name of the panel - {\em
	   i.e} the program's name.
	   @param bigsize Set {\em true} to use larger buttons.  {\em
	   false} uses buttons 75% smaller than the "large" size.
	   @param compact Set {\em true} to use abbreviated button labels
	   and short title and item label displays.  The default, {\em
	   false} uses full length labels.
	   @param useEject The default value of {\em true} uses a pixmap
	   of an eject button on the load/eject button.  {\em false} uses
	   a pixmap of a file folder instead.
	   @memo
	   */
	 AudioPanel( QWidget *parent, char *dn,
			   bool bigsize=true, 
			   bool compact=false, bool useEject=true );
	 /**
	   Class Destructor.
	   @memo
	   */
	 ~AudioPanel( void );

	 /**
	   Parts of the {\bf AudioPanel}, such as the track number,
	   volume meter, and time counter, have a different foreground
	   color than the rest of the panel.  This method sets that
	   color.
	   @memo
	   @see pDisplay
	   */
	 inline void setDisplayColor( QColor& col );

	 /**
	   Sets the color that the lock-light displays when the panel is
	   locked.
	   @memo
	   @see pControl
	   */
	 inline void setLockedColor( QColor& col );

	 /**
	   Sets the color that the lock-light displays when the panel is
	   unlocked.
	   @memo
	   @see pControl
	   */
	 inline void setUnlockedColor( QColor& col );

	 /**
	   Changes the font used to display the current time display
	   mode.  

	   The font I've chosen displays well at a small pointsize.  I've
	   also set the label widths based on this font and its size.
	   Changing the default using {\bf setTimeModeLabelFont} could
	   cause undesired results [clipped text, no text, etc.].

	   You need to make sure you choose a small enough pointsize so
	   that the text for the time display mode fits on the panel.

	   You shouldn't use this unless you're sure that the new font
	   you've chosen looks good at small pointsizes.
	   @memo
	   @see pDisplay
	   */
	 inline void setTimeModeLabelFont( QFont& f );

	 /**
	   Changes the font used to display the title and track labels.
	   Make sure the font you choose fits in the title and track
	   label widgets!
	   @memo
	   @see pDisplay
	   */
	 inline void setDisplayFont( QFont& f );

	 /**
	   Changes the global palette.
	   @memo
	   */
	 inline void changePalette( QPalette& p );

	 /**
	   Changes the global font [used for button labels and such].

	   As with all of the font-changing commands, make sure your new
	   font fits on the panel.
	   @memo
	   @see setTimeModeLabelFont, setDisplayFont
	   */
	 inline void changeFont( QFont& f);


	 /** 
	   Sets the panel's icon.  Not only calls the Qt-methods to do
	   so, but also calls some XLib routines to permit the use of
	   pixmaps with transparent regions!

	   If you compile your program using a version of Qt older than
	   v1.31, you need the following fix: the first item in {\em
	   *xpm}[]  must be the string "#/#* XPM *#/#".
	   @memo
	   */
	 void setPanelIcon( const char *xpm0[] );

	 /** 
	   Overloaded verison of {\bf setPanelIcon}(const char *[]).
	   @memo
	   */
	 void setPanelIcon( QPixmap& ipix );

	 /** 
	   Overloaded verison of {\bf setPanelIcon}(const char *[]).
	   @memo
	   */
	 void setPanelIcon( QImage& iimg );

	 /**
	   Connects the signals on several of the audio control buttons
	   to the corresponding slots on the \Ref{AudioDev} object, {\em
	   audiodev}.

	   You'll need to create new object that's a child of an
	   \Ref{AudioDev} object.
	   @memo
	   @see pAudio, pSpecial, AudioDev,
	   */
	 void pluginAudioControl( AudioDev *audiodev );

	 /**
	   Connects the signals on several display-controlling buttons to
	   the corresponding slots on the \Ref{AudioDev} object, {\em
	   audiodev}.

	   You'll need to create new object that's a child of an
	   \Ref{AudioDev} object.
	   @memo
	   @see pDisplay, AudioDev,
	   */
	 void pluginDisplayChange( AudioDev *audiodev );

	 /**
	   Connects the {\bg open}()slot of the specified \Ref{APDialog}
	   objects to the slots on the "Program", "Config.", and
	   "Catalog" buttons, respectively.

	   Any parameter with a null-value is ignored; no connection
	   takes place.  So, if your program doesn't use one of the
	   aforementioned buttons, feed the appropriate parameter a "0". 

	   I suggest that you create a \Ref{MaintainDB} object and feed
	   its pointer to the parameter {\em playlst_dlg}.
	   \Ref{MaintainDB} lets you use playlist databases, specifically
	   those used by {\tt WorkMan} and {\tt Xmcd}.

	   You'll probably want to create your own dialog [derived from
	   \Ref{APDialog}] for the "Config." button.
	   @memo
	   @see MaintainDB, Program
	   */
	 void pluginPanels( APDialog *prog_dlg=NULLDLG, 
					APDialog *config_dlg=NULLDLG,
					APDialog *playlst_dlg=NULLDLG );
	 //void createToolTips(void);
	 //void removeToolTips(void);

      //
   signals:
	 /**
	   {\em Signal}

	   Emitted when the Load/Eject button is clicked {\em and} the
	   lock button is off.
	   @memo [{\tt signal}]
	   @see bControl.  */
	 void loadeject();

	 /**
	   {\em Signal}

	   Emitted when the power button is clicked {\em and} the lock
	   button is off.
	   @memo [{\tt signal}]
	   @see bControl.
	   */
	 void quit();

	 
   private:
	 void pluginDefault(void);
	 void makePanelIcon(void);
	 void mousePressEvent( QMouseEvent *me );
	 void mouseMoveEvent( QMouseEvent *me );
	 //
	 APDialog nullmesg;
	 QWidget *itsParent;
	 pAudio *aud;
	 pControl *ctrl;
	 pSpecial *spec;
	 pDisplay *disp;
	 AudioDev *theDevPtr;
	 QPixmap icon_px;
	 QPoint pressOffset;
	 bool hasIcon,isSwapped;
	 char *default_name;

   private slots:
	 void doLoadEject();
	 void doPower();
	 void doIconify();
	 void toggleWTB(bool);
	 void newWinTitle(char *);
	 void swapTitleArtist();
	 void doMove(QPoint);
	 void pressedAt(QPoint);
};




//----------------------------------------------------------
//
// Inlined methods.
//
//----------------------------------------------------------




// Changes the panel colors
//
inline void AudioPanel::setDisplayColor( QColor& col )
{
   disp->setChildForegroundColor( col );
}
//
inline void AudioPanel::setLockedColor( QColor& col )
{
   ctrl->setLockedColor( col );
}
//
inline void AudioPanel::setUnlockedColor( QColor& col )
{
   ctrl->setUnlockedColor( col );
}


// Changes the global palette
//
inline void AudioPanel::changePalette(QPalette& p)
{
   setPalette(p);
   aud->changePalette(p);
   spec->changePalette(p);
   ctrl->changePalette(p);
   disp->changePalette(p);
}


// Changes the global font
//
inline void AudioPanel::changeFont(QFont& f)
{
   spec->changeFont(f);
   ctrl->changeFont(f);
}


// Changes the panel fonts
//
inline void AudioPanel::setDisplayFont( QFont& f )
{
   disp->changeDTextFont(f);
}
//
inline void AudioPanel::setTimeModeLabelFont( QFont& f )
{
   disp->changeTDMFont(f);
}





#endif // _AudioPanel_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
